"""Gimp Palette-loading

Loads a palette file and returns a list suitable for
Surface.setpalette()

If you've ever tried to specify a surface's entire palette using
surface.set_palette(), you may have noticed how frustrating it
is to create 256 individual RGB tuples. Kevin's code allows you
to load a gimp or Fractint palette file by
calling load_palette(filename), or you can specify your own
palette file in the format indicated in the docstrings.
"""
__author__ = "Kevin Turner"
__copyright__ = "Copyright (c) 2001 Kevin Turner"
__license__ = "Public Domain"
__version__ = "1.0"


import re

def load_palette(name='Firecode'):
    """Loads a palette file and returns a list suitable for Surface.setpalette()

    Loads a palette file, which is anything that looks like this:

    0 0 0
    0 0  4
    0 64 8
    ...

    GIMP and Fractint palette files both match this description.
    Lines that don't match are silently ignored.
    Only recognizes decimal values, not 0xFF or #AA66FF.
    """

    f = open(name)
    pal = []
    rgb_re = re.compile('^(?:\s+)?(\d+)\s+(\d+)\s+(\d+)')
    for l in f.readlines():
        m = rgb_re.search(l)
        if m:
            pal.append(tuple(map(int,m.groups())))

    if len(pal) != 256:
        print "Warning: palette \'%s\' does not contain 256 entries." % name
    if pal[0] != (0,0,0):
        print "Warning: palette \'%s\' has entry 0 set to non-black color"\
              % name, pal[0]
    return pal



"""Make system cursors from Images
Creates pygame cursors from a graphic file. This function allows you
to draw your custom cursors in an image-editing program, and keep them
all in a single file.

Your image file must be a multiple of 32x32 in size, and must use
certain colors: Black (0,0,0) and White (255,255,255) are treated
normally, Red (255, 0, 0) and Magenta (255, 0, 255) are used to
specify the hotspot, and any other color (green in the example) is
transparent.

Author website, http://nedbatchelder.com
"""
__author__ = "Ned Batchelder"
__copyright__ = "Copyright (c) 2001 Ned Batchelder"
__license__ = "Public Domain"
__version__ = "1.0"

import pygame

def MakeCursors(filename, names):
    """
    MakeCursors(filename, names):
    Read the image at filename, creating 32x32 cursors.  Return a dictionary
    of cursors, assigned to the names in names.
    Colors in the image:
    black and white are black and white.
    Red and magenta are the hotspot (colored black or white, respectively).
    Any other color is transparent.
    """

    # Load the raw tile image
    try:
        tiles = pygame.image.load(filename)
    except pygame.error:
        raise pygame.error, 'Could not load image "%s" %s' % (filename, pygame.get_error())

    tileWidth = 32
    tileHeight = 32
    tileSize = (tileWidth, tileHeight)

    curs = {}

    tiles.lock()

    # Split it into individual tiles
    for ycur in range(tiles.get_height() / tileHeight):
        for xcur in range(tiles.get_width() / tileWidth):
            tileX = xcur * tileWidth
            tileY = ycur * tileHeight

            strings = []
            hot = (0,0)

            if len(names) > 0 and names[0] != '':
                for y in range(tileY, tileY + tileHeight):
                    l = ''
                    for x in range(tileX, tileX + tileWidth):
                        clr = tiles.get_at((x, y))[:3]
                        # not sure why, but the colors are impure.  Force them to black, white, or green
                        pclr = map(lambda x: x > 100, clr)
                        if pclr == [0,0,0]:
                            l += 'X'
                        elif pclr == [1,1,1]:
                            l += '.'
                        elif pclr == [1,0,0]:
                            l += 'X'
                            hot = (x - tileX, y - tileY)
                        elif pclr == [1,0,1]:
                            l += '.'
                            hot = (x - tileX, y - tileY)
                        else:
                            l += ' '
                    strings.append(l)

                data = pygame.cursors.compile(strings, ".", "X")
                curs[names[0]] = ( (tileWidth, tileHeight), hot) + data

            names = names[1:]

    tiles.unlock()

    return curs


if __name__ == '__main__':
    pygame.init()
    print MakeCursors('cursors.gif', ('arrow', 'forward', 'left', 'right'))



#/usr/bin/env python
"""Load images from Zip Example

This shows how to load images from a zipfile. The script itself
will take the name of a zip file and display all the image inside
the zip.

You will want to change to code yourself to suit your game,
but this script shows you how to find images inside a zip file
and then load a specific image from the zip.

This example originally was included as an example with earlier
versions of pygame.

Many game projects require a very large number of graphics files -
sprite images, backgrounds, textures and other such resources. Some
developers simply ship their games with the graphics files as-is,
while others prefer to group them together in compressed archives.
While it won't keep your distribution size down (unless you're
using an uncompressed format like .bmp), python's zipfile module
makes it easy to ship your graphics resources in one easy-to
handle, familiar archive format.

The zipshow script will display each stored file in a zip
archive, one at a time - hit a key or click the mouse to page
through them, or hit ESC to quit. Place it in a directory with
an archive, or feed it the name of a zip file, and watch it do
its thing.
"""
__author__ = "Pete Shinners"
__copyright__ = "Copyright (c) 2001 Pete Shinners"
__license__ = "Public Domain"
__version__ = "1.0"


import zipfile, sys, glob, pygame
from cStringIO import StringIO
from pygame.locals import *



def showimage(imgsurface, name):
    "show a loaded image onto the screen"
    size = imgsurface.get_size()
    pygame.display.set_caption(name+'  '+`size`)
    screen = pygame.display.set_mode(size)
    screen.blit(imgsurface, (0,0))
    pygame.display.flip()

    #loop through events until done with this pic
    while 1:
        e = pygame.event.wait()
        if e.type == QUIT:
            return 0
        if e.type in (KEYDOWN, MOUSEBUTTONDOWN):
            break
    return 1


def zipslideshow(zipfilename):
    "loop through all the images in the zipfile"
    zip = zipfile.ZipFile(zipfilename)
    for file in zip.namelist():
        #get data from zipfile
        data = zip.read(file)

        #create stringio for data (a file-like object)
        data_io = StringIO(data)

        #load from a stringio object (ignore erroe exceptions)
        try: surf = pygame.image.load(data_io, file)
        except: continue

        #show image on the screen
        if not showimage(surf, file):
            return




def main():
    "run the program, handle arguments"
    pygame.init()
    zipfiles = sys.argv[1:]
    if not zipfiles:
        zipfiles = glob.glob('*.zip') #find any zipfile in this directory
    if not zipfiles:
        raise SystemExit('No ZIP files given, or in current directory')
    zipslideshow(zipfiles[0])


#run the script if not being imported
if __name__ == '__main__': main()




#! /usr/bin/env python
"""Word-wrapped text display module

This module contains one function: render_textrect, which will
try to return a surface containing the supplied string,
word-wrapped as necessary to fit within the supplied rectstyle
object. The function can also left or right justify, or center
the supplied text. It will raise a TextRectException if the
string won't fit into the rect's area.

Tries to render the supplied text string in the passed rect,
by word-wrapping where possible.  Interprets an embedded \n
as a linefeed.  Displays the final text left or right
justified, or centered.
"""
__author__ = "David Clark"
__copyright__ = "Copyright (c) 2001 David Clark"
__license__ = "Public Domain"
__version__ = "1.0"



class TextRectException:
    def __init__(self, message = None):
        self.message = message
    def __str__(self):
        return self.message

def render_textrect(string, font, rect, text_color, background_color, justification=0):
    """Returns a surface containing the passed text string, reformatted
    to fit within the given rect, word-wrapping as necessary. The text
    will be anti-aliased.

    Takes the following arguments:

    string - the text you wish to render. \n begins a new line.
    font - a Font object
    rect - a rectstyle giving the size of the surface requested.
    text_color - a three-byte tuple of the rgb value of the
                 text color. ex (0, 0, 0) = BLACK
    background_color - a three-byte tuple of the rgb value of the surface.
    justification - 0 (default) left-justified
                    1 horizontally centered
                    2 right-justified

    Returns the following values:

    Success - a surface object with the text rendered onto it.
    Failure - raises a TextRectException if the text won't fit onto the surface.
    """

    import pygame

    final_lines = []

    requested_lines = string.splitlines()

    # Create a series of lines that will fit on the provided
    # rectangle.

    for requested_line in requested_lines:
        if font.size(requested_line)[0] > rect.width:
            words = requested_line.split(' ')
            # if any of our words are too long to fit, return.
            for word in words:
                if font.size(word)[0] >= rect.width:
                    raise TextRectException, "The word " + word + " is too long to fit in the rect passed."
            # Start a new line
            accumulated_line = ""
            for word in words:
                test_line = accumulated_line + word + " "
                # Build the line while the words fit.
                if font.size(test_line)[0] < rect.width:
                    accumulated_line = test_line
                else:
                    final_lines.append(accumulated_line)
                    accumulated_line = word + " "
            final_lines.append(accumulated_line)
        else:
            final_lines.append(requested_line)

    # Let's try to write the text out on the surface.

    surface = pygame.Surface(rect.size)
    surface.fill(background_color)

    accumulated_height = 0
    for line in final_lines:
        if accumulated_height + font.size(line)[1] >= rect.height:
            raise TextRectException, "Once word-wrapped, the text string was too tall to fit in the rect."
        if line != "":
            tempsurface = font.render(line, 1, text_color)
            if justification == 0:
                surface.blit(tempsurface, (0, accumulated_height))
            elif justification == 1:
                surface.blit(tempsurface, ((rect.width - tempsurface.get_width()) / 2, accumulated_height))
            elif justification == 2:
                surface.blit(tempsurface, (rect.width - tempsurface.get_width(), accumulated_height))
            else:
                raise TextRectException, "Invalid justification argument: " + str(justification)
        accumulated_height += font.size(line)[1]

    return surface


if __name__ == '__main__':
    import pygame
    import pygame.font
    from pygame.locals import *

    pygame.init()

    display = pygame.display.set_mode((400, 400))

    my_font = pygame.font.Font(None, 22)

    my_string = "Hi there! I'm a nice bit of wordwrapped text. Won't you be my friend? Honestly, wordwrapping is easy, with David's fancy new render_textrect() function.\nThis is a new line.\n\nThis is another one.\n\n\nAnother line, you lucky dog."

    my_rect = pygame.Rect((40, 40, 300, 300))

    rendered_text = render_textrect(my_string, my_font, my_rect, (216, 216, 216), (48, 48, 48), 0)

    if rendered_text:
        display.blit(rendered_text, my_rect.topleft)

    pygame.display.update()

    while not pygame.event.wait().type in (QUIT, KEYDOWN):
        pass






"""Resource Caching Module

Most game projects require graphics, sound and other resources to be
loaded multiple times - a sprite's image might be loaded for each
instance of that sprite. Normally, you'll write some sort of resource
management system to ensure that all these identical resources aren't
kept in memory at once.

This module wraps pygame's resource loading functions.  It ensures that only
one copy of any given resource will be in memory at any time - further requests
for that same resource will get a reference to the loaded copy.  This behavoir
can be overridden by specifying that the resource loading be forced - all
clients of that resource will then refer to the new version.  The resource can
be manually removed from memory with the clear_* set of functions.

This code might be useful if a game is going to be loading many copies of a
given disk resource.  Most of the time, you'd write your own cache; now you
can just use this one.

Remember to call the 'set path' functions before trying to load anything;
otherwise, you'll get a ValueError.
"""
__author__ = "David Clark"
__copyright__ = "Copyright (c) 2001 David Clark"
__license__ = "Public Domain"
__version__ = "1.0"


import pygame, os

__images = {}
__fonts = {}
__sounds = {}

__images_path = None
__fonts_path = None
__sounds_path = None

def get_image(filename, force_reload = 0):
    """
    get_image(filename, force_reload = 1) --&gt; surface
    Call this function instead of pygame.image.load - it will load the image
    from the disk the first time, then just return a reference to the copy each
    subsequent time.  This function does no colorkey setting or pixel format
    conversion; you'll have to do that manually, if you wish.
    """

    if not __images_path:
        raise ValueError, "resources.set_images_path() not called yet."
    if (force_reload == 1 or filename not in __images.keys()):
        try:
            surface = pygame.image.load(os.path.join(__images_path, filename))
        except pygame.error:
            raise IOError, "File " + filename + " not found."
        __images[filename] = surface
        return surface
    else:
        return __images[filename]

def has_image(filename):
    """
    has_image(filename) --&gt; Boolean
    Returns true if the image is in memory, false if it has to be loaded from
    disk.
    """
    return __images.has_key(filename)

def clear_image(filename):
    """
    clear_image(filename) --&gt; Boolean
    Eliminates the image from memory.  Subsequent calls will load it from the
    disk.  Returns True if the resource was found in memory, False if it
    Wasn't.  Use this to reduce the memory footprint, if you're sure you won't
    be needing the resource again.
    """

    try:
        del __images[filename]
        return 1
    except KeyError:
        return 0


def get_font(filename, size, force_reload = 0):
    """
    get_font(filename, size, force_reload = 1) --&gt; surface
    Call this function instead of pygame.font.Font - it will load the font
    from the disk the first time, then just return a reference to the copy each
    subsequent time.
    """
    if not __fonts_path:
        raise ValueError, "resources.set_fonts_path() not called yet."
    if (force_reload == 1 or filename not in __fonts.keys()):
        try:
            font = pygame.font.Font(os.path.join(__fonts_path, filename), size)
        except pygame.error:
            raise IOError, "File " + filename + " not found."
        __fonts[filename] = font
        return font
    else:
        return __fonts[filename]

def has_font(filename):
    """
    has_font(filename) --&gt; Boolean
    Returns true if the font is in memory, false if it has to be loaded from
    disk.
    """
    return __fonts.has_key(filename)

def clear_font(filename):
    """
    clear_font(filename) --&gt; Boolean
    Eliminates the font from memory.  Subsequent calls will load it from the
    disk.  Returns True if the resource was found in memory, False if it
    Wasn't.  Use this to reduce the memory footprint, if you're sure you won't
    be needing the resource again.
    """
    try:
        del __fonts[filename]
        return 1
    except KeyError:
        return 0

def get_sound(filename, force_reload = 0):
    """
    get_sound(filename, force_reload = 1) --&gt; sound
    Call this function instead of pygame.mixer.Sound - it will load the sound
    from the disk the first time, then just return a reference to the copy each
    subsequent time.
    """
    if not __sounds_path:
        raise ValueError, "resources.set_sounds_path() not called yet."
    if (force_reload == 1 or filename not in __fonts.keys()):
        try:
            sound = pygame.mixer.Sound(os.path.join(__sounds_path, filename))
        except pygame.error:
            raise IOError, "File " + filename + " not found."
        __sounds[filename] = sound
        return sound
    else:
        return __sounds[filename]

def has_sound(filename):
    """
    has_sound(filename) --&gt; Boolean
    Returns true if the sound is in memory, false if it has to be loaded from
    disk.
    """
    return __sounds.has_key(filename)

def clear_sound(filename):
    """
    clear_sound(filename) --&gt; Boolean
    Eliminates the sound from memory.  Subsequent calls will load it from the
    disk.  Returns True if the resource was found in memory, False if it
    Wasn't.  Use this to reduce the memory footprint, if you're sure you won't
    be needing the resource again.
    """
    try:
        del __sounds[filename]
        return 1
    except KeyError:
        return 0

def set_images_path(path):
    """
    set_images_path(path) --&gt; Boolean
    Set the path you'll be loading the images off of.  Pass the string
    representation of the new path.  Raises an exception if the path doesn't
    exist, otherwise it returns True.
    """
    if not os.access(path, os.F_OK):
        raise IOError, path + " not found."
    if path.endswith(os.sep):
        path = path[:-1]
    global __images_path
    __images_path = path
    return 1

def set_fonts_path(path):
    """
    set_fonts_path(path) --&gt; Boolean
    Set the path you'll be loading the fonts off of.  Pass the string
    representation of the new path.  Raises an exception if the path doesn't
    exist, otherwise it returns True.
    """
    if not os.access(path, os.F_OK):
        raise IOError, path + " not found."
    if path.endswith(os.sep):
        path = path[:-1]
    global __fonts_path
    __fonts_path = path
    return 1

def set_sounds_path(path):
    """
    set_sounds_path(path) --&gt; Boolean
    Set the path you'll be loading the sounds off of.  Pass the string
    representation of the new path.  Raises an exception if the path doesn't
    exist, otherwise it returns True.
    """
    if not os.access(path, os.F_OK):
        raise IOError, path + " not found."
    if path.endswith(os.sep):
        path = path[:-1]
    global __sounds_path
    __sounds_path = path
    return 1


def get_images_path():
    """
    get_images_path() --&gt; String
    Returns the current value of the images path, or None if it hasn't been
    set yet.
    """
    return __images_path

def get_fonts_path():
    """
    get_fonts_path() --&gt; String
    Returns the current value of the fonts path, or None if it hasn't been
    set yet.
    """
    return __fonts_path

def get_sounds_path():
    """
    get_sounds_path() --&gt; String
    Returns the current value of the sounds path, or None if it hasn't been
    set yet.
    """
    return __sounds_path



"""Subclassable Rect for Older Python

UserRect, python class wrapped around the pygame Rect type. This allows
you to do things like inherit the rectangle object into a sprite class. While
this makes for some neat features, current testing has shown there is a bit of
a peformance penalty. (as opposed to just keeping a Rect value inside the class)

Note, this might be worthless now, as python-2.2 allows you to subclass real
python types, no need for this placeholder. Still, it may provide some assistance
to somebody out there.

Before the class/type unification that was included in Python 2.2, data
types implemented in C (such as pygame's Rect class) weren't considered
true classes - they couldn't be subclassed. Pete included the UserRect
module in the pygame distribution to get around that - it's just the
thinnest possible python wrapper around Rect. I dont think anyone ever
used it for anything, but in case someone misses it, here it is. It
might be handy for a game like Breakout, where everything is
rectangular, and you could create a simple sprite class as
"class Sprite(UserRect):". However, it would be cleaner and faster
just to upgrade to python 2.2, and subclass the actual Rect class.
"""
__author__ = "Pete Shinners"
__copyright__ = "Copyright (c) 2001 Pete Shinners"
__license__ = "Public Domain"
__version__ = "1.0"


from pygame import Rect


class UserRect:
    """UserRect() -> class instance
Python class for the pygame Rect type

This python class can be treated exactly like a normal Rect
object. The only difference is it is a real python class
object, not a C extension type.
"""
    def __init__(self, *args):
        try: self.__dict__['rect'] = apply(Rect, args)
        except TypeError:
            raise TypeError, 'Argument must be rectstyle object'
        for a in dir(self.rect):
            self.__dict__[a] = getattr(self.rect, a)

    def __getattr__(self, name):
        return getattr(self.rect, name)
    def __setattr__(self, name, val):
        if name is 'rect':
            self.__dict__['rect'][:] = val
        else:
            try: setattr(self.__dict__['rect'], name, val)
            except (AttributeError, KeyError): self.__dict__[name] = val
    def __len__(self): return 4
    def __getitem__(self, i): return self.rect[i]
    def __setitem__(self, i, val): self.rect[i] = val
    def __getslice__(self, i, j): return self.rect[i:j]
    def __setslice__(self, i, j, val): self.rect[i:j] = val
    def __nonzero__(self): return nonzero(self.rect)
    def __cmp__(self, o): return cmp(self.rect, o)
    def __repr__(self):
        return '<UserRect(%d, %d, %d, %d)>' % tuple(self.rect)
    def __str__(self):
        return '<UserRect(%d, %d, %d, %d)>' % tuple(self.rect)




#! /usr/bin/env python
"""Zoomable Surface Class

Traditionally, it has been impossible to subclass fundamental
classes like pygame's Surface class. This has meant that if you
wanted to implement a varient of the Surface class, you had to
wrap it; this took more effort and introduced overhead for each
method call. With Python 2.2, however, you can inherit from base
classes, and with February 8th's CVS commit, you can subclass
the Surface and Rect classes.

You can use the ZoomSurface just like a regular surface, following
the instructions mentioned in the comments below. ZoomSurfaces
have one additional power - the 'output' drawing can larger or
smaller than the actual surface dimensions. For example, yur
surface might be 320 x 240, and you could have it drawn at
640 x 480, so small details can be more easily seen. Normally,
you'd have to do a pygame.transform call yourself to accomplish
this, but ZoomSurfaces do it automatically.

Two classes, ZoomScreen and ZSurface, designed to make debugging in
detail (graphically) easier.  I think the second one shows more
promise, but Python doesn't like it, even v2.2. The first one works,
though, even if it is considerably less elegant.

Dependencies:  Requires pygame.transform, and will require
Python 2.2 if and when the subclassing is fixed.
"""
__author__ = "David Rooney"
__copyright__ = "Copyright (c) 2002 David Rooney"
__license__ = "Public Domain"
__version__ = "1.0"


import pygame
from pygame import HWSURFACE, RESIZABLE, ASYNCBLIT, OPENGL, HWPALETTE, DOUBLEBUF, FULLSCREEN, RLEACCEL
from pygame import display
from pygame.transform import scale, rotozoom
from math import tan, pi


class ZoomScreen:
	"""
	"""
	def __init__(self, size, source = None, opts = DOUBLEBUF):
		"""		ZoomScreen( size [,source [,options]] )   -->   None

		This is intended to be the main screen, the one usually returned
		by display.set_mode()
		Therefore, it takes mostly the same arguments as set_mode()does, with
		the optional addition of a specified source surface.

		The source surface defaults to None, which is really useless unless you
		set_source() soon after...

		This was written mainly with the intent of making small-scale graphics
		work visible while debugging, although I think it could also be used
		to show a miniature of a large surface, instead.  (eg: an inset map of
		the whole world while one sector of it is on screen)

		Possibly useless technical note:  if given a tuple, this function will
		use the transform.scale() function, or if given a scaling factor, will
		use tranform.rotozoom() (with rotation==0)
		"""
		pygame.init()
		try:
			if len(size) == 2:
				self.size = size
			elif len(size) == 4:
				self.size == size[-2:]
			else:
				self.size = size[:2]
		except TypeError:
			self.size = float(size)
		self.img = display.set_mode( self.size, opts )
		if source:
			self.source = source
		else:
			self.source = Surface( (0,0) )

	def set_source(self, src):
		if type(src) == pygame.SurfaceType:
			self.source = src
		self.draw()
		return

	def draw(self):
		"""		ZoomScreen.draw()   -->  None
		"""
		try:
			len(self.size)
			self.img.blit( scale(self.source, self.size),(0,0) )
		except TypeError:
			self.img.blit( rotozoom(self.source, 0, self.size),(0,0) )
		return



class ZSurface( pygame.Surface ):
	"""		Mostly like any other Surface, with a few extra methods for zoom-related
	actions.  Scales the ZSurface to any arbitrary size requested, when drawing.

	The ZSurface itself remains a constant size, acting just like a normal Surface;
	it only scales its draw() output.

	New member variables are:
		zoomsize 				# most recent requested size
		_aspect_ratio			# (read-only) current aspect ratio

	Any Surface inquiries having to do with size will answer with actual sizes,
	not zoomed ones.
	"""

	def __init__(self, *argv):
		Surface.__init__(self, argv)
		self.zoomsize = None
		if self.get_height() != 0:
			self._aspect_ratio = float(self.get_width()) / float(self.get_height())
		else:
			self._aspect_ratio = None

	def get_aspect_ratio(self):
		"""		ZSurface.get_aspect_ratio()  -->  float or None

		This method returns this ZSurface's aspect ratio, which is its width divided
		by its height ( w/h ).  If height is zero, (ie, divide-by-zero trouble) the
		aspect ratio is redefined as None.
		"""
		return self._aspect_ratio

	def check_ratio(self, src, acceptable = 0.1):
		"""		ZSurface.check_ratio( src [, acceptable_distortion] )  -->  int

		Designed as a small convenience, this method will compare the aspect
		ratios of this ZSurface and the given src Surface.  If the difference
		between them is a float within an acceptable range, 0.1 (10%) by
		default, this returns a 1, giving back a 0 if the distortion is too
		great.

		If either Surface has an aspect ratio of None, it's certainly a bad
		match, so it will simply return 0.

		This function is best called _before_ blitting a Surface to this one.
		Otherwise, you could mistakenly blit a 10x75 Surface into, say, a
		320x480 ZSurface, with appropriately disasterously ugly stretchy
		results.
		However, squeezing a 180x200 Surface into a 170x190 ZSurface shouldn't
		work out too badly (it's less than 1% distortion).

		Note that this function won't prevent you from doing such a (possibly)
		foolish thing, if you want to.  Its sole purpose is to check beforehand.
		"""
		if self._aspect_ratio == None:
			return 0
		else:
			if is_instance( src, ZSurface ):
				othrAR = ZSurface.get_aspect_ratio()
				if othrAR == None:
					return 0
			else:
				x, y = src.get_size()
				if y== 0 or x==0:
					return 0
				else:
					othrAR = float(x)/float(y)
			ratio_diff = self._aspect_ratio - othrAR
			if abs( ratio_diff ) <= acceptable:
				return 1
			else:
				return 0

	def draw(self, (wide,high)=(0,0) ):
		"""		ZSurface.draw( [ (width,height)=(0,0) ] )    -->  Surface

		This function is for output only.  Big surprize.

		Called without arguments, it will return a copy of this ZSurface
		in its last known dimensions.  That is, if you haven't ever resized it,
		it will return in original size, but if you asked it to draw itself at,
		say, 320x200, each subsequent call will return a Surface of 320x200,
		until you specify otherwise.

		This method will optionally take 1 argument, a length==2 sequence to
		designate the desired return size (which will then be its default size).

		In the future, I think I'll also make it support RectType arguments, but
		it doesn't yet.  Sorry.
		"""
		if (wide == 0) and (high == 0):
			if self.zoomsize:
				zsize = self.zoomsize
			else:
				zsize = self.get_size()
		elif (wide == 0) or (high == 0):
			if self.zoomsize:
				if wide == 0:
					wide == self.zoomsize[0]
				else:
					high == self.zoomsize[1]
				self.zoomsize = [wide,high]
			else:
				if wide == 0:
					wide == self.get_width()
				else:
					high == self.get_height()
				self.zoomsize = [wide,high]
			zsize = self.zoomsize
		else:
			self.zoomsize = [wide,high]
			zsize = None
		if zsize:
			return scale( self, zsize )
		else:
			return self

























































"""Box Selection Tool

Manages mouse handling for box selection with rubberbanded box drawing.
The module handles the mouse events, and returns the area selected by
the mouse when the mouse is released. It also draws an rubberband
style outline ofthe area selected. The module includes a simple test
program,running the module allows you to test it out.

This is a simple and ready-to-use class which adds C&C (Command & Conquer)
like selection rectangles to your games/apps so that the user can resize
a rectangle to cover some area on the screen (f.ex. to choose all units in
that area)

You can run this sample right away using the supplied main() method if you
want to check the effect.

A short description on the usage of this class:
1) Initiate the class passing it the surface to display the rectangle on,
   the starting point of your selection rectangle and an optional color
   in which to draw the rectangle (defaults to black)
2) Check for MOUSEMOTION and then call updateRect() and draw() methods (this
   is doing the main work)
3) After the user released the mousebutton (or whenever you want actually)
   make a final call to updateRect() (it will return the selection rectangle
   and you might want to save that in some var ;).
4) Don't forget to call the hide() method to clear the final selection
   rectangle (unless you're totally redrawing the screen anyways)

Possible customizations:
1) the default drawing color (line X)
2) use another surfarray method (pixels3d is used here which only works
   for surfaces with 32 or 24 bpp - shouldn't be that hard)
3) something else I didn't think of :)

Dependencies:
1) pygame w/ surfarray support
2) PyNumeric
"""
__author__ = "Frank Raiser"
__copyright__ = "Copyright (c) 2001 Frank Raiser"
__license__ = "Public Domain"
__version__ = "1.0"


import pygame
from pygame.locals import *
import Numeric,copy

class SelectionRect:
    """ class SelectionRect
    utility class for using selection rectangles
    """
    def __init__(self,screen,start,col=(0,0,0)):
        """ __init__(self,screen,start,col=(0,0,0))
        Constructor. Pass starting point of selection rectangle in 'start'
        and color value in which the selection rectangle shall be drawn
        in 'col'
        """
        self.start         = start
        self.col           = col
        self.oldrect       = start[0],start[1],1,1
        tmp                = screen.get_at((start[0],start[1]))[:3]
        self.screen_backup = [[tmp],[tmp],[tmp],[tmp]]

    def updateRect(self,now):
        """ updateRect(self,now) -> rect tuple
        This returns a rectstyle tuple describing the selection rectangle
        between the starting point (passed to __init__) and the 'now' edge and
        updates the internal rectangle information for correct drawing.
        """
        x,y = self.start
        mx,my = now
        if mx < x:
            if my < y:
                self.rect = mx,my,x-mx,y-my
            else:
                self.rect = mx,y,x-mx,my-y
        elif my < y:
            self.rect = x,my,mx-x,y-my
        else:
            self.rect = x,y,mx-x,my-y
        return self.rect

    def draw(self,screen):
        """ draw(self,screen)
        This hides the old selection rectangle and draws the current one
        """
        # just some shortcuts :P
        surf = pygame.surfarray.pixels3d(screen)
        r    = self.rect
        # hide selection rectangle
        self.hide(screen)

        # update background information
        self.screen_backup[0] = copy.copy(surf[r[0]:r[0]+r[2],r[1]])
        self.screen_backup[1] = copy.copy(surf[r[0]:r[0]+r[2],r[1]+r[3]-1])
        self.screen_backup[2] = copy.copy(surf[r[0],r[1]:r[1]+r[3]])
        self.screen_backup[3] = copy.copy(surf[r[0]+r[2]-1,r[1]:r[1]+r[3]])

        # draw selection rectangle:
        surf[r[0]:r[0]+r[2],r[1]]        = self.col
        surf[r[0]:r[0]+r[2],r[1]+r[3]-1] = self.col
        surf[r[0],r[1]:r[1]+r[3]]        = self.col
        surf[r[0]+r[2]-1,r[1]:r[1]+r[3]] = self.col

        self.oldrect = r

        pygame.display.update(r)

    def hide(self,screen):
        """ hide(self,screen)
        This hides the selection rectangle using the stored background
        information. You usually call this after you're finished with the
        selection to hide the last rectangle.
        """
        surf = pygame.surfarray.pixels3d(screen)
        x,y,x2,y2 = self.oldrect[0],self.oldrect[1],\
                    self.oldrect[0]+self.oldrect[2],\
                    self.oldrect[1]+self.oldrect[3]
        surf[x:x2,y   ] = self.screen_backup[0]
        surf[x:x2,y2-1] = self.screen_backup[1]
        surf[x,   y:y2] = self.screen_backup[2]
        surf[x2-1,y:y2] = self.screen_backup[3]
        pygame.display.update(self.oldrect)

def main():
    """ main()
    Simple test program for showing off the selection rectangles
    """
    pygame.init()
    screen = pygame.display.set_mode((640,480),0,24)

    # create a dotted background
    surf = pygame.surfarray.pixels3d(screen)
    surf[:] = (255,255,255)
    surf[::4,::4] = (0,0,255)
    pygame.display.update()

    # make up a test loop
    finished = 0
    selection_on = 0
    while not finished:
        for e in pygame.event.get():
            if e.type == QUIT or (e.type == KEYDOWN and e.key == K_ESCAPE):
                finished = 1
            elif e.type == MOUSEBUTTONDOWN and e.button == 1:
                if not selection_on:
                    # begin with selection as the user pressed down the left
                    # mouse button
                    selection_on = 1
                    selection = SelectionRect(screen,e.pos)
            elif e.type == MOUSEMOTION:
                if selection_on:
                    # update the selection rectangle while the mouse is moving
                    selection.updateRect(e.pos)
                    selection.draw(screen)
            elif e.type == MOUSEBUTTONUP and e.button == 1:
                if selection_on:
                    # stop selection when the user released the button
                    selection_on = 0
                    rect = selection.updateRect(e.pos)
                    # don't forget this!
                    # (or comment it out if you really want the final selection
                    #  rectangle to remain visible)
                    selection.hide(screen)
                    # just FYI
                    print "Final selection rectangle:",rect

    pygame.quit()

if __name__ == '__main__': main()



"""Fast Digit Rendering

This is code is for fast rendering of integers.

This class prerenders the integers from 0-9 in a font and color
you specify. You can then feed it a number between 0 and
9,999,999, and it will quickly fetch those images and render
your number in the font you specified. Mr. Feuer has predefined
a few colors and font sizes for you, or you can use the default
font and size of 30.
"""
__author__ = "Adam Feuer"
__copyright__ = "Copyright (c) 2001 Adam Feuer"
__license__ = "Public Domain"
__version__ = "1.0"


import os
import pygame
from pygame.locals import *
import game, gfx

FONTSIZE_TINY   = 17
FONTSIZE_SMALL  = 35
FONTSIZE_MEDIUM = 50
FONTSIZE_LARGE  = 80

GREY      = ( 50, 150, 150)
BLUE_GREY = (114, 124, 193)
WHITE     = (255, 255, 255)
GREEN     = (123, 204,  83)
WHITE     = (255, 255, 255)

FONT = None
FONTSIZE = 30


class FastDigits:
    """Build an array of digit images, and provide a speedy method for converting an integer to an image...
       speeds up digit rendering by about 10x. You can get another 2-3x by using an image/rectangle cache.

    Usage:

    fd= FastDigits()
    digits_image, digits_rect = fd.num2image(frames_per_second)

    """
    def __init__(self,fontpath=None,fontsize=FONTSIZE,fontcolor=WHITE):
        # build digit table
        self.digits = []
        font = pygame.font.Font(fontpath,fontsize)
        for num in range(0,10):
            d_img,d_rect = gfx.text(font,fontcolor,"%d"%num)
            self.digits.append([d_img,d_rect])
        self.digitwidth = self.digits[0][1].width
        self.digitheight = self.digits[0][1].height

    def getdigits(self,num,zeropad=0):
        """convert a number to sequence of digits- handles integers up to 9,999,999;
             zeropad is number of digits to zeropad"""
        digitseq = []

        if zeropad > 0:
            fmt = "%" + ("0.%dd" % zeropad)
        else:
            fmt = "%d"

        s = fmt % num
        if len(s)>7:
            s = s[-7:]
        for c in s:
            digitseq.append(ord(c)-ord('0'))
        return digitseq

    def num2image(self,num,zeropad=0):
        """convert an integer to an image - pad image with zeros to zeropad places
             returns image, image.get_rect()"""
        digitseq = self.getdigits(num,zeropad)
        dsurface = pygame.Surface((self.digitwidth * len(digitseq), self.digitheight))
        bgd = 0,0,0
        dsurface.set_colorkey(bgd)

        x = 0
        for d in digitseq:
            dsurface.blit(self.digits[d][0],(x,0))
            x += self.digits[d][1].width
        return dsurface.convert(),dsurface.get_rect()





"""2d Vector Class

Any game involving 2D physics could do with a class similar to this
one. This class is as fast as possible while still providing all
the functionality you would commonly need.

Note that storing the vector as cartesian coordinates (rather than
polar or both) makes for faster for vector addition. It is assumed
that you won't often need the polar coordinates.

Here's a small class that provides the basics of 2d vector math.
Adding other functions like vector multiplication and dot product
calculation should be simple if you follow the outline of this class.

Author website, http://grace.2ya.com
"""
__author__ = "Chris Wood"
__copyright__ = "Copyright (c) 2002 Chris Wood"
__license__ = "Public Domain"
__version__ = "1.0"


from math import sin, cos, atan, hypot

# The quick way to know how big the pie is
PI = 3.141592653589793238462643383
TwoPI = PI * 2.0
HalfPI = PI * 0.5
OneAndHalfPI = PI * 1.5

# new math function
def direction(x, y):
    """Return the direction component of a vector (in radians), given
    cartesian coordinates.
    """
    if x > 0:
        if y >= 0:
            return atan(y / x)
        else:
            return atan(y / x) + TwoPI
    elif x == 0:
        if y > 0:
            return HalfPI
        elif y == 0:
            return 0
        else:
            return OneAndHalfPI
    else:
        return atan(y / x) + PI


class Vector:
    """Store a vector in cartesian coordinates."""

    def __init__(self, f = 0.0, mag = 0.0):
        """Create a Vector, given polar coordinates."""
        # Calculate cartesian coordinates
        self.x = mag * cos(f)
        self.y = mag * sin(f)

    def add(self, b):
        """Add b to self, where b is another Vector."""
        self.x += b.x
        self.y += b.y

    def heading(self):
        """Return the direction of the Vector in radians."""
        return direction(self.x, self.y)

    def mag(self):
        """Return the magnitude of the Vector."""
        return hypot(self.x, self.y)



"""Bitmap Font Module

A simple class for bitmapped fonts in pygame. A bitmapped font is
simply a font specified as a bitmap. Bitmapped fonts are easy to
use, fast, and let you display fancy multicoloured text. The
characters are all the same size, so it's easy to treat the
screen as a matrix of character cells, as if you were in text
mode (e.g. for a status screen between levels). Drawbacks are
that you must of course draw the font yourself, you can't change
the colour, and you don't get special effects like bold, italics,
etc.

This submission is so well documented, I don't have much to add. As Mr. Sidorsky says, this file package allows you to create your own bitmapped font image, define it with an index file, then blit it to your surface. The included code uses xmodelines(), which is only available in Python 2.1


bmpfont lets you define where each character is within the bitmap,
along with some other options.	This lets you use a bitmap of any
dimension with characters of any size.	The file where the position
of each character is defined is called the font index file.  It is
a simple text file that may contain the lines listed below.
Whitespace is ignored, but the keywords are case-sensitive.
NOTE:  Blank lines and comments are not allowed!

bmpfile filename
- filename is the full name (with optional path) of the bitmap file
  to use.  The file can be of any type supported by pygame.image.
  Defaults to font.bmp with no path if omitted.

width x
height y
- Specifies the dimensions of a character, in pixels.  Each
  character must be of the same width and height.  width defaults
  to 8 and height to 16 if omitted.

transrgb r g b
- Specifies the colour being used to indicate transparency.  If you
  don't wish to use transparency, set this to an unused colour.
  Defaults to black (0, 0, 0) if omitted.

alluppercase
- If present, indicates the font only has one case of letters which
  are specified in the index using upper case letters.	Strings
  rendered with BmpFont.blit() will be converted automatically.  If
  omitted, the font is assumed to have both cases of letters.

- All other lines are treated as character index specifiers with
  the following format:

  char x y

    - char is the character whose position is being specified.  It
        can also be "space" (without quotes) to define a position for
        the space character.
    - x is the column number where char is located.  The position
        within the bitmap will be x * width (where width is specified
        above).
    - y is the row number where char is located.	The position will
        be y * height.
"""



import pygame
from pygame.locals import *

__all__ = ["BmpFont"]

class BmpFont:
	"""Provides an object for treating a bitmap as a font."""

	# Constructor - creates a BmpFont object.
	# Parameters:  idxfile - Name of the font index file.
	def __init__(self, idxfile = "bmpfont.idx"):
		# Setup default values.
		self.alluppercase = 0
		self.chartable = {}
		self.bmpfile = "bmpfont.bmp"
		self.width = 8
		self.height = 16
		self.transrgb = (0, 0, 0)

		# Read the font index.	File errors will bubble up to caller.
		f = open(idxfile, "r")

		for x in f.xreadlines():
			# Remove EOL, if any.
			if x[-1] == '\n': x = x[:-1]
			words = x.split()

			# Handle keywords.
			if words[0] == "bmpfile":
				self.bmpfile = x.split(None, 1)[1]
			elif words[0] == "alluppercase":
				self.alluppercase = 1
			elif words[0] == "width":
				self.width = int(words[1])
			elif words[0] == "height":
				self.height = int(words[1])
			elif words[0] == "transrgb":
				self.transrgb = (int(words[1]), int(words[2]),
								 int(words[3]))
			else:  # Default to index entry.
				if words[0] == "space": words[0] = ' '
				if self.alluppercase: words[0] = words[0].upper()
				self.chartable[words[0]] = (int(words[1]) * self.width,
											int(words[2]) * self.height)
		f.close()

		# Setup the actual bitmap that holds the font graphics.
		self.surface = pygame.image.load(self.bmpfile)
		self.surface.set_colorkey(self.transrgb, RLEACCEL)

	# blit() - Copies a string to a surface using the bitmap font.
	# Parameters:  string	 - The message to render.  All characters
	#						   must have font index entries or a
	#						   KeyError will occur.
	#			   surf 	 - The pygame surface to blit string to.
	#			   pos		 - (x, y) location specifying location
	#						   to copy to (within surf).  Meaning
	#						   depends on usetextxy parameter.
	#			   usetextxy - If true, pos refers to a character cell
	#						   location.  For example, the upper-left
	#						   character is (0, 0), the next is (0, 1),
	#						   etc.  This is useful for screens with
	#						   lots of text.  Cell size depends on the
	#						   font width and height.  If false, pos is
	#						   specified in pixels, allowing for precise
	#						   text positioning.
	def blit(self, string, surf, pos = (0, 0), usetextxy = 1):
		"""Draw a string to a surface using the bitmapped font."""
		x, y = pos
		if usetextxy:
			x *= self.width
			y *= self.height
		surfwidth, surfheight = surf.get_size()
		fontsurf = self.surface.convert(surf)

		if self.alluppercase: string = string.upper()

		# Render the font.
		for c in string:
			# Perform automatic wrapping if we run off the edge of the
			# surface.
			if x >= surfwidth:
				x -= surfwidth
				y += self.height
				if y >= surfheight:
					y -= surfheight

			surf.blit(fontsurf, (x, y),
					 (self.chartable[c], (self.width, self.height)))
			x += self.width

# Example code.  Run this file as a script to activate.
if __name__ == "__main__":
	import pygame
	pygame.init()
	screen = pygame.display.set_mode((400, 300), 0, 16)
	# Assumes font.idx and font.bmp are in the current directory.
	bmpfont = BmpFont()
	msg = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"\
		  "0123456789 !@#$%^&*()-=_+\|[]{};:'\",.<>/?`~" * 5
	bmpfont.blit(msg, screen)
	while pygame.event.poll().type != QUIT: pass

# End of file bmpfont.py



#!/usr/bin/env python
"""Embed Image In Python Code

The function's docstring is very clear - it describes both the uses
and limitations of this utility. Basically, using this function, you
can generate a compressed string version of an image file, and store
that string in your program. This is useful if you have a python
script that (for example) only uses one small image, and you don't
want to ship two separate files. This script incorporates a simple
image viewer into the generated string, so you can view the
resulting image.

image2py will take an image file, and embed it into a regular .PY file.
the image is converted to a string and then compressed and converted to
a base64 string. the image will also preserve its colorkey and palette.

note that the script created comes with a small amount of bootstrap code
to display the image embedded in the file. (just in case you later need
to determine what all those long strings really look like) :]

this is mainly a handy utility. it doesn't really useful in all situations.
but in cases where you create your own small utility module for pygame, you
might want to use small bitmapped images. instead of requiring external
resources, you can cut and paste the python code to create the image into
your script.

obviously you'll want to keep the image size within reason. anything bigger
than 100x100 is going to create a sizeable PY file.
"""
__author__ = "Pete Shinners"
__copyright__ = "Copyright (c) 2001 Pete Shinners"
__license__ = "Public Domain"
__version__ = "1.0"

import sys, os
import pygame, zlib, base64


packed_image_file = r"""#!/usr/bin/env python
import base64, zlib, pygame, pygame.image
pygame.display.init()
size = %r
type = %r
imagedata = ''.join((
%s
))
image = pygame.image.fromstring(zlib.decompress(base64.decodestring(imagedata)),size, type)
%s
if __name__ == '__main__':
    pygame.init()
    s = pygame.display.set_mode(image.get_size())
    image.set_colorkey()
    s.blit(image, (0,0))
    pygame.display.flip()
    while 1:
        if pygame.event.wait().type in \
           (pygame.QUIT,pygame.KEYDOWN): break
"""


def embedimage(imagename, outfilename):
    img = pygame.image.load(imagename)

    type = 'RGB'
    if img.get_bytesize() == 1:
        type = 'P'
    elif img.get_masks()[3]:
        type = 'RGBA'

    imgstring = pygame.image.tostring(img, type)
    imgcompressed = zlib.compress(imgstring, 9)
    encoded = base64.encodestring(imgcompressed)

    instructions = ''
    if img.get_bitsize() == 8:
        pal = img.get_palette()
        palstr = ''
        for x in range(0, len(pal), 6):
            palstr += '  ' + repr(pal[x:x+6])[1:-1] + ',\n'
        instructions += 'palette = (%s)\n' % palstr
        instructions += 'image.set_palette(palette)\n'
    if img.get_colorkey():
        instructions += 'image.set_colorkey(%r)\n' % (img.get_colorkey()[:3],)
    if img.get_masks()[3]:
        instructions += 'image.set_alpha(255)\n'
    elif img.get_alpha():
        instructions += 'image.set_alpha(%i)\n' % img.get_alpha()

    file = open(outfilename, 'w')
    prettydata = ',\n'.join(map(repr, (encoded.split())))
    file.write(packed_image_file%(img.get_size(), type, prettydata, instructions))




if __name__ == '__main__':
    if len(sys.argv) &lt; 3:
        print 'USAGE:', sys.argv[0], '<imagename> <outfile.py>'
        sys.exit()
    embedimage(sys.argv[1], sys.argv[2])




"""Graphical Cursor Class

This is a nice little GfxCursor class that gives you arbitrary
mousecursor loadable from all SDL_image supported filetypes.

The example script included with this class is pretty neat -
it allows you to draw as in a paint program, over a multi-coloured
background. Note that the graphical cursor (a simple green circle
in this case) reverts to the standard pygame cursor whenever
drawing is taking place. The idea of using a software-generated
artificial cursor, rather than pygame's standard cursor, is an
especially good one if you're considering using hardware
acceleration (pygame.HWSURFACE) - the standard cursor may
flicker or disappear altogether on a hardware surface. It would
be pretty trivial to subclass gfxcursor to make an animated
cursor - just create an "animate()" method, and call that from
update(). Also note that the cursor's constructor function takes
a simple surface as an argument, so the image could be generated
with pygame.draw functions as it is here, loaded from an image
file, or even created with pygame.font.


Usage:
Instantiate the GfxCursor class. Either pass the correct parameters
to the constructor or use setCursor, setHotspot and enable later on.

The blitting is pretty optimized, the testing code at the bottom
of this script does a pretty thorough test of all the drawing
cases. It enables and disables the cursor, as well as uses a
changing background.

In your mainloop, the cursor.show() should be what you draw last
(unless you want objects on top of the cursor?). Then before drawing
anything, be sure to call the hide(). You can likely call hide()
immediately after the display.flip() or display.update().

The show() method also returns a list of rectangles of what needs
to be updated. You can also move the cursor with pygame.mouse.set_pos()

That's it. Have fun with your new funky cursors.
"""
__author__ = "Frank Raiser and Pete Shinners"
__copyright__ = "Copyright (c) 2002 Frank Raiser"
__license__ = "Public Domain"
__version__ = "1.0"


import pygame

class GfxCursor:
    """
    Replaces the normal pygame cursor with any bitmap cursor
    """

    def __init__(self,surface,cursor=None,hotspot=(0,0)):
        """
        surface = Global surface to draw on
        cursor  = surface of cursor (needs to be specified when enabled!)
        hotspot = the hotspot for your cursor
        """
        self.surface = surface
        self.enabled = 0
        self.cursor  = None
        self.hotspot = hotspot
        self.bg      = None
        self.offset  = 0,0
        self.old_pos = 0,0

        if cursor:
            self.setCursor(cursor,hotspot)
            self.enable()

    def enable(self):
        """
        Enable the GfxCursor (disable normal pygame cursor)
        """
        if not self.cursor or self.enabled: return
        pygame.mouse.set_visible(0)
        self.enabled = 1

    def disable(self):
        """
        Disable the GfxCursor (enable normal pygame cursor)
        """
        if self.enabled:
            self.hide()
            pygame.mouse.set_visible(1)
            self.enabled = 0

    def setCursor(self,cursor,hotspot=(0,0)):
        """
        Set a new cursor surface
        """
        if not cursor: return
        self.cursor = cursor
        self.hide()
        self.show()
        self.offset = 0,0
        self.bg = pygame.Surface(self.cursor.get_size())
        pos = self.old_pos[0]-self.offset[0],self.old_pos[1]-self.offset[1]
        self.bg.blit(self.surface,(0,0),
            (pos[0],pos[1],self.cursor.get_width(),self.cursor.get_height()))

        self.offset = hotspot

    def setHotspot(self,pos):
        """
        Set a new hotspot for the cursor
        """
        self.hide()
        self.offset = pos

    def hide(self):
        """
        Hide the cursor (useful for redraws)
        """
        if self.bg and self.enabled:
            return self.surface.blit(self.bg,
                (self.old_pos[0]-self.offset[0],self.old_pos[1]-self.offset[1]))

    def show(self):
        """
        Show the cursor again
        """
        if self.bg and self.enabled:
            pos = self.old_pos[0]-self.offset[0],self.old_pos[1]-self.offset[1]
            self.bg.blit(self.surface,(0,0),
                (pos[0],pos[1],self.cursor.get_width(),self.cursor.get_height()))
            return self.surface.blit(self.cursor,pos)

    def update(self,event):
        """
        Update the cursor with a MOUSEMOTION event
        """
        self.old_pos = event.pos



if __name__ == '__main__': #test it out
    import pygame.draw
    pygame.init()
    screen = pygame.display.set_mode((400, 300))
    screen.fill((50, 50, 111), (0, 0, 400, 150))
    pygame.display.flip()
    pygame.display.set_caption('Test the GfxCursor (and paint)')

    image = pygame.Surface((20, 20))
    pygame.draw.circle(image, (50, 220, 100), (10, 10), 8, 0)
    pygame.draw.circle(image, (220, 200, 50), (10, 10), 8, 2)
    image.set_at((9, 9), (255,255,255))
    image.set_colorkey(0, pygame.RLEACCEL)

    magicbox = pygame.Rect(10, 10, 100, 90)
    magiccolor = 0

    cursor = GfxCursor(screen, image, (10, 10))
    finished = 0
    downpos = None
    while not finished:
        dirtyrects = []
        dirtyrects.extend([cursor.hide()])
        for e in pygame.event.get():
            if e.type in (pygame.QUIT, pygame.KEYDOWN):
                finished = 1
                break
            elif e.type == pygame.MOUSEBUTTONDOWN:
                cursor.disable()
                downpos = e.pos
            elif e.type == pygame.MOUSEBUTTONUP:
                cursor.enable()
                downpos = None
            elif downpos and e.type == pygame.MOUSEMOTION:
                r = pygame.draw.line(screen, (100,100,100), downpos, e.pos, 2)
                dirtyrects.append(r)
                downpos = e.pos
                cursor.update(e)
            elif not downpos and e.type == pygame.MOUSEMOTION:
                cursor.update(e)

        magiccolor = (magiccolor + 2) % 255
        r = screen.fill((0, 0, magiccolor), magicbox)
        dirtyrects.append(r)

        #here's how we sandwich the flip/update with cursor show and hide
        dirtyrects.extend([cursor.show()])
        pygame.display.update(dirtyrects)

        pygame.time.delay(5) #should be time.wait(5) with pygame-1.3 :]



#!/usr/bin/env python2
"""Interface to the high score file.

This provides an interface for the loading and saving the game's high
score file.  High score files are generally placed in a location shared
between all the players on that machine, but not writable by them (on
filesystems which support file permissions).  For the game to be able
to write to the score file, it must be run from a setgid wrapper.

All the fun is in the ScoreFile class.

There's also a write_wrapper_source subroutine, see its documentation
for information about the need for a set-group-id (SGID) wrapper.

NOTE: You should be warned that if run SGID, this module will change
the process EGID upon import.

This code pickles a machine-wide high-score listing to (by default)
/var/games. The high-score file contains username, realname, date,
score/level, version and custom info fields, and should be created
by your installation script. In order to get around permissions
issues under unix, scorefile.py will create a C source file for
you, which you must then compile and install properly yourself
- read the write_wrapper_source docstring carefully. This code
should work with little modification on non-unix filesystems.
"""
__author__ = "Kevin Turner"
__copyright__ = "Copyright (c) 2001 Kevin Turner"
__license__ = "Public Domain"
__version__ = "1.0"



TRUE = 1 == 1
FALSE = not TRUE

import os
import time
import pickle
import sys
from types import *

# Try importing pygame, so we can define some rendering methods.
try:
    import pygame
except ImportError:
    pass

# According to the Filesystem Hierarchy Standard (2.2 beta),
# /var/games is the place to put score files.
# XXX - What's the default path for non-Unix systems?
default_dir = os.path.normpath('/var/games')

__methods__ = ['write_wrapper_source']

class ScoreFileError:
    def __init__(self, errorstring):
        self.value = errorstring
    def __str__(self):
        return `self.value`

class ScoreRecord:
    """An individual record in the score file.

    score: A numeric value indicating the player's score.

    level: The number of the highest level or wave reached by the player.
           May be "None" if the game is not structured by level.

    login: The identity of this user, according to the OS,
           e.g. "agreensp".

    name: The name by which the player wishes to be known,
          e.g. "Alan Greenspan" or "MoneyMan".

    version: A string describing the version, or release, of the game
             software.

    date: The date and time when the score was recorded.  Stored in UTC,
          as a tuple.

    extradata: A dictionary object containing game-specific stats, e.g.
               board size, difficulty level, character race/class, etc.
               You may put anything in here you like, as long as it's
               pickleable.
    """

    __members__ = ['date','login','name',
                   'score','level', 'extradata']
    __methods__ = ['get_date']

    def __init__(self, score, name=None, level=None, extradata=None):
        self.date = time.gmtime(time.time())

        # XXX - If we're a network game server, os.getlogin() probably
        # doesn't return what we want.
        try:
            self.login = os.getlogin()
        except:
            self.login = None

        self.score = score
        self.name = name
        self.level = level

        self.extradata = extradata

    def get_date(self, formatstring=None):
        """Returns a string describing the time of the entry, in local time.

        formatstring is the same format as used by time.strftime()."""

        if not formatstring:
            formatstring = '%c'

        formatstring = str(formatstring)

        # self.date is a UTC time in a tuple.
        utc_seconds = time.mktime(self.date)

        # I thought I would have to test for time.daylight here,
        # and then use either time.altzone or time.timezone accordingly,
        # but apparently one of the other time functions already handles it,
        # because if I use altzone, I'm over-correcting by an hour.
        local_offset = -time.timezone

        local_seconds = utc_seconds + local_offset

        local_time = time.localtime(local_seconds)

        time_string = time.strftime(formatstring, local_time)

        return time_string

    def __getattr__(self,name):
        "Check extradata to see if has this attribute."
        if name != 'extradata' and self.extradata \
           and self.extradata.has_key(name):
            return self.extradata[name]
        raise AttributeError

    def __str__(self):
        ds = {'score':self.score,
              'name':self.name,
              'level':self.level,
              'login':self.login,
              'date':self.get_date()}

        if self.extradata:
            ds.update(self.extradata)

        return str(ds)
# ScoreRecord

class ScoreFile:
    """A high score file which may be read from and added to.

    Entries are stored sorted by score."""

    __members__ = ['filename', 'game_name', 'game_version', 'records']
    __methods__ = ['add', 'write', 'crop', 'top', 'latest']

    # TODO: Provide a default DRAW method to render some scores to a SDL surface.

    def __init__(self, game_name, game_version, filename=None, dirname=None):
        """Load or create a score file.

        game_name: this name is stored in the score file, and possibly used to
                   generate a filename if none is supplied.
        game_version: this is the current version of the game, stored with each
                      entry.
        filename: the name of the high score file.  If none is given, an attempt
                  is made to construct one from game_name.
        dirname: the directory where the high score file should be placed, if
                 the filename is not specified as an absolute pathname.
                 The Filesystem Hierarchy Standard dictates that this be
                 /var/games.
        """
        self.game_name = str(game_name)
        self.game_version = str(game_version)


        if dirname:
            dirname = os.path.normpath(dirname)
        else:
            dirname = default_dir

        if not os.path.abspath(dirname):
            dirname = os.path.join(default_dir,dirname)

        if filename:
            filename = os.path.normpath(filename)
        else:
            # find a score file, or make a filename up (based on
            # game_name), or get one from the configuration module
            # XXX - How should this work on non-Unix platforms?
            # XXX - configuration module does not exist

            # XXX - filter game_name so it contains only
            #  filesystem-friendly characters.
            filename = self.game_name + '.hiscore'

        if not os.path.isabs(filename):
            filename = os.path.join(dirname, filename)

        # This sort of thing *should* be done by the game's installer.
        # We'll make a bumbling attempt to try and get something working,
        # though.
        if not os.path.exists(os.path.dirname(filename)):
            if _obtain_permission():
                os.makedirs(os.path.dirname(filename),0775)
            else:
                os.makedirs(os.path.dirname(filename))

        if not os.path.exists(filename):
            _obtain_permission()
            if os.access(os.path.dirname(filename), os.W_OK):
                # we can create a scorefile, but for now it's empty.
                self.records = []
            else:
                raise ScoreFileError, \
                      "I can't write to directory %s to create the "\
                      "score file." % os.path.dirname(filename)
            _drop_permission()
        else:
            _obtain_permission()
            fobj = open(filename, 'r')
            try:
                loaded_scorefile = pickle.load(fobj)
                self.records = loaded_scorefile.records
                del loaded_scorefile
            except EOFError:
                # We get this if the file is zero-sized.
                self.records = []
            fobj.close()
            _drop_permission()

        self.filename = filename
    # ScoreFile.__init__()

    def add(self, score, name=None, level=None, extradata=None, write=TRUE):
        """Make a new entry in the high score file.

        See ScoreRecord for a description of the arguments.
        Returns the entry's place in the high score list, with
        1 being the highest.

        write: Should the score file be written to disk after this add?
               If 'must', then raise an exception if the write fails.
               Otherwise, if true, attempt the write (but only print a
               warning if it fails), if false, add the entry to the score
               file in memory but make no attempt to save to disk.
        """
        # XXX - doc string should do necessary translcusion of ScoreRecord
        # argument docs, don't make the reader jump around to do our work.

        score = float(score)
        name = str(name)

        score_rec = ScoreRecord(score, name, level, extradata)

        score_rec.game_version = self.game_version

        pos = self.place(score)

        self.records.insert(pos-1, score_rec)

        if(write):
            if write=='must':
                self.write()
            else:
                try:
                    self.write()
                except IOError:
                    print "WARNING: failed to save scorefile, got %s: %s"\
                          % sys.exc_info()[:2]
        # endif(write)
        return pos
    # ScoreFile.add()

    def place(self,score):
        """Returns the place that this score would earn in the list."""

        if (not self.records) or score >= self.records[0].score:
            # New High Score!
            pos = 0
        elif score < self.records[-1].score:
            # Congratz -- Yours is the lowest score yet.
            pos = len(self.records)
        else:
            # binary search to find the insertion location
            high_end = len(self.records) - 1
            low_end = 0
            pos = 0
            while abs(high_end - low_end) > 1:
                pos = int(round((high_end + low_end) / 2))
                # Reminder: A HIGHER score means a LOWER index.
                if(score >= self.records[pos].score):
                    high_end = pos
                else:
                    low_end = pos
            # whend
        # fi
        return pos + 1
    # ScoreFile.place()

    def write(self):
        """Save the score file to disk.

        Note: ScoreFile.add() calls this method, so you should
        rarely need to do so yourself."""
        assert(self.filename)

        if _obtain_permission():
            # Err.  We can't mask the write bit for the file's owner,
            # otherwise this user won't be able to write to it in the future!
            # Ideally, the file has already been created (by the install process),
            # and so is already owned by someone else.
            os.umask(02)
        fobj = open(self.filename, 'w')
        pickle.dump(self, fobj)
        fobj.close()
        _drop_permission()

    def crop(self, bytes): # XXX - Untested!
        """Limits the size of the scorefile.

        This method may be used to limit the size of the scorefile.
        It will remove as many entries from the scorefile as are
        required to make it fit in the specified number of bytes.
        The entries removed are those with the lowest scores.

        Returns the number of entries trimmed."""

        bytes = long(bytes)
        assert(bytes > 0)

        trimmed_entries = 0

        while 1:
            pickledfile = pickle.dumps(self)

            fat = len(pickledfile) - bytes

            if fat <= 0:
                return trimmed_entries
            else:
                if len(self.records) == 0:
                     raise ScoreFileError, "Failed to make the score "\
                           "file small enough for you."
                del self.records[-1]
                trimmed_entries = trimmed_entries + 1
        # whend
    # ScoreFile.crop()

    def __get_item__(self, key):
        """Scores are returned sorted highest to lowest."""
        return self.records[key]

    def get_by_time(self, key):
        """Like get_item, but sorted by time, not score."""
        time_list = map(None, self.records[:].date, range(len(self.records)))

        time_list.sort()

        time_sorted = [ self.records[tl[1]] for tl in time_list ]

        return time_sorted[key]
    # ScoreFile.get_by_time()

    def top(self, n=1):
        """Returns a list of the top n scores (defaulting to 1).

        Returns a list of ScoreRecords.
        Note: This object supports the standard slice syntax,
        so this method is equivalent to my_scorefile[:n]."""

        return self[:n]

    def latest(self, n=1):
        """Returns the n most recent entries in the score file.

        If n is omitted, it defaults to 1.
        Returns a list of ScoreRecords, starting with the most recent."""

        results = self.get_by_time(-n)
        results.reverse()
        return results

    def __len__(self):
        """The number of entries in the high score file."""
        return len(self.records)

    def __str__(self):
        s = "%s: Score File for %s, contains %d entries.\n" %\
            (self.filename, self.game_name, len(self.records))
        for i in self.records:
            sr = "%d\t%s\t%s\t%s\t%s" \
                 % (i.score, i.level, i.name, i.login, i.get_date())
            if i.extradata:
                sr = sr + str(i.extradata)
            s = s + sr + "\n"
        return s
    # ScoreFile.__str__()
# ScoreFile


def _obtain_permission():
    """Re-gain our SGID group identity."""
    if _SGID_WRAPPER:
        os.setregid(_user_gid,_privledged_gid)
        return TRUE
    return FALSE

def _drop_permission():
    """Drop our SGID group identity for the time being."""
    if _SGID_WRAPPER:
        os.setregid(_privledged_gid,_user_gid)
        return TRUE
    return FALSE

# top-level code
if os.environ.has_key('RUN_BY_WRAPPER'):
    _privledged_gid = os.getegid()
    _user_gid = os.getgid()
    if _privledged_gid != _user_gid:
        _SGID_WRAPPER = TRUE
    else:
        _SGID_WRAPPER = FALSE
        print "Apparently run from a wrapper, but it doesn't seem to be SGID."
    # XXX: This causes us to drop our SGID permissions upon importing
    # the module!  If our parent module was not expecting those to
    # change, it may not be happy with us.
    _drop_permission()
else:
    _SGID_WRAPPER = FALSE
    print "Warning: Not run from a SGID wrapper, I may use inappropriate"
    print " ownership and/or permissions for the high score file."
    # TODO:  At this point, if we believe there *should* be a wrapper installed,
    #  but we just weren't run by it, we could exec it here, if we wanted to.
# top-level code

def write_wrapper_source(path,source_file):
    """Writes C source code that exec's path to source_file.

    On some platforms (e.g. unix) it's desirable to have a shared score
    file between users.  This requires that the game be able to write to
    the score file no matter who's playing, but we'd rather not make the
    file world-writable.

    If the score file is installed as owned by a "games" group, then
    the game could write to that file if it were owned by the games
    group and had its sgid bit set (chmod g+s).  Unfortunately for
    us, most systems don't use sgid for interpreted files (e.g. your
    python script), so we need to make a "real executable" to put the
    sgid bit on.  This, having gained the "games" permissions, will
    in turn run our python script.

    This function writes the source for such a wrapper program to
    a file.  You must then see to it that the file is compiled and
    properly installed."""

    path = os.path.normpath(path)
    source_file = os.path.normpath(source_file)

    if not os.path.exists(path):
        print "Warning: path",path,"doesn't seem to exist."

    fobj = open(source_file, 'w')

    s = '#include <unistd.h>\n#include <stdlib.h>\n'
    s = s + "/* Compile like: gcc -o my_wrapper %s */\n" % source_file
    s = s + """
            int main(int argc, char* argv[])
            {
            putenv("RUN_BY_WRAPPER=1");
            """
    fobj.write(s)
    fobj.write("return execv(\"%s\",argv);\n}\n" % path)
    fobj.close()
# write_wrapper_source()
